//
// SMAC.hh       : SMAC/MoteNIC Radio Support Include File
// Authors       : Fabio Silva
//
// Copyright (C) 2000-2002 by the University of Southern California
// $Id: SMAC.hh,v 1.1 2002/12/11 01:03:25 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef _SMAC_HH_
#define _SMAC_HH_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 

#include "main/iodev.hh"

// The following structures come from smac_msg.h and phy_radio_msg.h

typedef struct PhyHeader {
  unsigned char length;
} PhyHeader_;

typedef struct MacHeader {
  PhyHeader_ phyHdr;
  char type;
  short toAddr;
  short fromAddr;
  unsigned short duration;
  unsigned char fragNo;
} MacHeader_;

typedef struct AppHeader {
  MacHeader_ macHdr;
  uint8_t appType;
} AppHeader_;

#define DIFFUSION_PACKET_TYPE  0x3A
#define SMAC_BROADCAST 0xffff
#define MAX_PKT 200

// SMAC tos device files
#define SMAC_MOTENIC_DEVICE "/dev/mote/1/tos"

class RFIdCacheEntry {
public:
  RFIdCacheEntry(int32_t diffusion_id, short radio_id) :
    diffusion_id_(diffusion_id), radio_id_(radio_id)
  {};
  int32_t diffusion_id_;
  short radio_id_;
};

typedef list<RFIdCacheEntry *> RFIdCache;

class SMAC : public DiffusionIO {
public:
  SMAC();
  DiffPacket recvPacket(int fd);
  void sendPacket(DiffPacket pkt, int len, int dst);

protected:
  RFIdCache RfCache_;
  void addCache(int32_t diffusion_id, short radio_id);
  bool findInCache(int32_t diffusion_id, short *radio_id);
  DiffPacket assemblePacket(short radio_id, int len, DiffPacket pkt);
};

#endif // !_SMAC_HH_
