//
// emstar.hh     : Various relevant emstar include files put together
// Authors       : Compiled by Fabio Silva
//
// Copyright (C) 2000-2003 by the University of Southern California
// $Id: emstar_misc.hh,v 1.1 2003/02/28 07:37:01 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include <sys/types.h>
#include <inttypes.h>
#include <sys/ioctl.h>

// From local_types.h
typedef uint32_t node_id_t;
typedef uint32_t if_id_t;

typedef struct loc {
  float x, y, z;
} loc_t;

// From sync.h
typedef enum {
  UNKNOWN = 0,
  SYNTHETIC = 1,
  PROCESSOR = 2,
  CODEC = 3,
  MOTE = 4,
  ZERO = 5,
  DEBUG = 6,
  CODEC2 = 7
} comp_id_t;

typedef struct sync_id_t_s {
  node_id_t node;		/* ID of the node */
  comp_id_t comp:16;		/* component */
  int index:16;			/* for padding, and convenience */
} sync_id_t;

/* "remote timestamp" structure */
typedef struct remote_ts {
  sync_id_t timebase;
  struct timeval stamp;
  uint32_t sum_errors;
} remote_ts_t;

// From link.h
/*
 * Please use care when adding to the link_pkt_type list -- if someone
 * else is using the same number as you (and hasn't checked their code
 * in yet), you can see a conflict between your code and theirs.  You
 * might want to use a "random" number (up to 255) instead of the next
 * number in the sequence.
 */
typedef enum link_pkt_type {
  PKT_TYPE_ALL = 0,		/* do not use!! */
  PKT_TYPE_PING = 1,		/* ping request/response protocol */
  PKT_TYPE_HB = 2,		/* heartbeats for neighbor discovery */
  PKT_TYPE_DQT = 3,
  PKT_TYPE_FLOOD = 4,		/* flooding algorithm wrapper */
  PKT_TYPE_GLS = 5,
  PKT_TYPE_FLOOD_GLS = 6,

  PKT_TYPE_DIFFUSION = 15,	/* directed diffusion */

  PKT_TYPE_RNC = 126,		/* rnc algorithm wrapper */
  PKT_TYPE_RNC_TEST = 127	/* rnc test driver packet type */
} link_pkt_type_t;

/*
 * This is the "universal" link interface header -- the header used to
 * speak to almost any kind of packet-transport interface in the
 * system.  The over-the-air format will be different depending on the
 * type of network; this is just the struct used to talk to the driver.
 */
typedef struct link_pkt {
  union {
    if_id_t id;			/* source interface address */
    loc_t loc;
  } src;
  union {
    if_id_t id;			/* destination interface address */
    loc_t loc;
  } dst;

  /*
   * time this packet was received.  if_rcv_time is the raw timeval
   * generated by the network interface, e.g. the mote clock.
   * rcv_time is that time, after conversion (using the timesync
   * service) to a plain local-clock timeval.  If timesync is not
   * available, rcv_time is an inaccurate best guess - a
   * gettimeofday() at the time the packet arrived.
   */
  struct timeval rcv_time;
  remote_ts_t if_rcv_time;

  link_pkt_type_t type:8;	/* protocol number (8 bits) */
  uint8_t flags;		/* space for flags, currently unused */
  uint8_t max_hops;		/* max number of hops, if routed */
  uint8_t pad3;			/* unused */
  uint8_t data[0];		/* pointer to the data that follows */
} link_pkt_t;

#define LINK_BROADCAST ((if_id_t) 0xFFFFFFFF)

// From packet_client.h
#define PD_MAX_FILTER_LEN 128

typedef struct pd_filter {
  int len;
  char data[PD_MAX_FILTER_LEN];
} pd_filter_t;

/*
 *  ioctl() numbers
 */

#define PD_IOCTL_TYPE 0x8b

// query the queue to see how big the first element is
#define PD_GET_FRONT_SIZE  _IOR(PD_IOCTL_TYPE, 0x1, int)

// 0x2: not here
// 0x3: not here

// gets and sets the loopback mode
#define PD_SET_LOOP_MODE   _IOW(PD_IOCTL_TYPE, 0x4, int)
#define PD_GET_LOOP_MODE   _IOR(PD_IOCTL_TYPE, 0x4, int)

// gets and sets the generic filter
#define PD_SET_FILTER      _IOW(PD_IOCTL_TYPE, 0x5, pd_filter_t)
#define PD_GET_FILTER      _IOR(PD_IOCTL_TYPE, 0x5, pd_filter_t)

// gets and sets the qlen for the outgoing queue
#define PD_SET_OUTQ_LEN    _IOW(PD_IOCTL_TYPE, 0x6, int)
#define PD_GET_OUTQ_LEN    _IOR(PD_IOCTL_TYPE, 0x6, int)

// gets and sets the qlen for the incoming queue
#define PD_SET_INQ_LEN     _IOW(PD_IOCTL_TYPE, 0x7, int)
#define PD_GET_INQ_LEN     _IOR(PD_IOCTL_TYPE, 0x7, int)
