/*

  rf_ioctl.h

  Header file for the WINSNG 2 Radio Low Level (ioctl) API
  
  -------------------------------------------------------------------------
  
  Copyright (c) 2002, Sensoria Corporation.  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:
  
  * Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
  
  * Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  * Neither the name of the Sensoria Corporation nor the names of
  its contributors may be used to endorse or promote products derived
  from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS
  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
  IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  
*/   


#ifndef __RF_IOCTL_H__
#define __RF_IOCTL_H__
 
/*Allow linkage to C or C++*/
__BEGIN_DECLS

/*
 *  packet filter type
 */ 

#define RF_MAX_FILTER_LEN 128

typedef struct rf_filter {
  int len;
  char data[RF_MAX_FILTER_LEN];
} rf_filter_t;

/*
 *  ioctl() numbers
 */

#define RF_IOCTL_TYPE 0x8b

// query the queue to see how big the first element is
#define RF_GET_FRONT_SIZE  _IOR(RF_IOCTL_TYPE, 0x1, int)

// 0x2: not here
// 0x3: not here

// gets and sets the loopback mode
#define RF_SET_LOOP_MODE   _IOW(RF_IOCTL_TYPE, 0x4, int)
#define RF_GET_LOOP_MODE   _IOR(RF_IOCTL_TYPE, 0x4, int)

// gets and sets the generic filter
#define RF_SET_FILTER      _IOW(RF_IOCTL_TYPE, 0x5, rf_filter_t)
#define RF_GET_FILTER      _IOR(RF_IOCTL_TYPE, 0x5, rf_filter_t)

// gets and sets the qlen for the outgoing queue
#define RF_SET_OUTQ_LEN    _IOW(RF_IOCTL_TYPE, 0x6, int)
#define RF_GET_OUTQ_LEN    _IOR(RF_IOCTL_TYPE, 0x6, int)

// gets and sets the qlen for the incoming queue
#define RF_SET_INQ_LEN     _IOW(RF_IOCTL_TYPE, 0x7, int)
#define RF_GET_INQ_LEN     _IOR(RF_IOCTL_TYPE, 0x7, int)


/* Allow linkage to C or C++ */
__END_DECLS

#endif




